/*
    Copyright (c) 2021 digitalSTROM

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/
#include "bonjour.h"

#include <ds-avahi/client.h>
#include <ds-avahi/publish.h>

#include "propertysystem.h"

static ds::LogChannel dsLogChannel("bonjour");

namespace dss {
namespace {
struct BonjourImpl : ds::asio::Resource<>, Bonjour {
  Apartment &m_apartment;
  dsAvahi::Client m_client{io()};
  dsAvahi::EntryGroup m_entryGroup{m_client, dsAvahi::CollisionMode::ALTERNATIVE_NAME};
  ds::Slot<> DS_UNIQUE_NAME(slot){*this, m_apartment.changed(), [this]() { updatePublishedServices(); }};

  BonjourImpl(ds::asio::ResourceBase &parent, Apartment &apartment)
      : ds::asio::Resource<>{parent, "bonjour"}, m_apartment{apartment} {
    updatePublishedServices();
  }

  void updatePublishedServices() {
    const auto port = ds::narrow<uint16_t>(
        m_apartment.getPropertySystem()->getIntValue("/config/subsystems/WebServer/announcedport"));
    const auto name = m_apartment.getName();
    m_entryGroup.setServices(std::vector<dsAvahi::PublishedService>{
        dsAvahi::PublishedService{
            .name = name, .type = "_dssweb._tcp", .port = port, .txts = {ds::str("apartment=", m_apartment.id())}},
        dsAvahi::PublishedService{.name = name, .type = "_http._tcp", .port = port},
    });
  }
};

} // namespace
std::unique_ptr<Bonjour> makeBonjour(ds::asio::ResourceBase &parent, dss::Apartment &apartment) {
  return std::make_unique<BonjourImpl>(parent, apartment);
}

} // namespace dss
