/*
    Copyright (c) 2019  digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
    
*/
#pragma once

#include <ds-model/groups.h>
#include <ds-model/scenarios.h>

#include <trompeloeil.hpp>

#include "apartment-proxy.h"
#include "model/cluster.h"

namespace dss {

class ApartmentProxyMock : public trompeloeil::mock_interface<ApartmentProxy> {
public:
  MAKE_CONST_MOCK4(callScene, void(const Dsuid&, int, bool, ds::DateTime));
  MAKE_MOCK5(callScene, void(int, int, int, bool, ds::DateTime));
  IMPLEMENT_CONST_MOCK2(saveScene);
  MAKE_CONST_MOCK3(callSceneMin, void(const Dsuid&, int, ds::DateTime));
  MAKE_CONST_MOCK4(callSceneMin, void(int, int, int, ds::DateTime));
  MAKE_CONST_MOCK3(undoScene, void(const Dsuid&, int, ds::DateTime));
  MAKE_MOCK4(undoScene, void(int, int, int, ds::DateTime));
  MAKE_CONST_MOCK3(setOutputValue, void(const Dsuid&, uint8_t, ds::DateTime));
  MAKE_CONST_MOCK4(setOutputValue, void(int, int, uint8_t, ds::DateTime));
  MAKE_CONST_MOCK4(setOutputValue, void(const Dsuid&, ChannelType, double, ds::DateTime));
  IMPLEMENT_CONST_MOCK3(setSceneLocalPrio);
  IMPLEMENT_CONST_MOCK2(setOperationLock);
  MAKE_CONST_MOCK4(stopDimming, void(int, int, int, ds::DateTime));
  MAKE_CONST_MOCK1(stopDimming, void(const Dsuid&));
  IMPLEMENT_CONST_MOCK1(completeMoving);
  IMPLEMENT_MOCK0(stopDsmTimer);
  MAKE_CONST_MOCK6(updateDSMState, void(Group&, int, int, int, int, int));
  MAKE_CONST_MOCK2(updateDSMState, void(Cluster&, int));
  MAKE_CONST_MOCK2(syncWithDsm, void(int, int));
  MAKE_CONST_MOCK1(syncWithDsm, void(std::shared_ptr<Cluster>));

  IMPLEMENT_CONST_MOCK1(zoneScenarioStatus);
  IMPLEMENT_CONST_MOCK1(dsDeviceStatus);
  IMPLEMENT_CONST_MOCK1(lightsStatus);
  IMPLEMENT_CONST_MOCK1(shadesStatus);
  IMPLEMENT_CONST_MOCK1(audioStatus);
  IMPLEMENT_CONST_MOCK0(ventilationUnitStatus);
  IMPLEMENT_CONST_MOCK1(ventilationStatus);
  IMPLEMENT_CONST_MOCK1(recirculationStatus);
  IMPLEMENT_CONST_MOCK0(groupsStatus);
  IMPLEMENT_CONST_MOCK2(scenarioDeviceList);

  MAKE_CONST_MOCK3(write, void(const Dsuid&, std::string, std::string));
  MAKE_CONST_MOCK3(write, void(const Dsuid&, std::string, double));
  MAKE_CONST_MOCK3(write, void(const Dsuid&, std::string, bool));

  IMPLEMENT_CONST_MOCK2(updateSceneConfig);
  IMPLEMENT_CONST_MOCK2(updateDeviceSceneChannelProperty);

  IMPLEMENT_CONST_MOCK4(updateDeviceConfig);
  IMPLEMENT_CONST_MOCK1(getDeviceThresholds);

  IMPLEMENT_CONST_MOCK0(ventilationUnitLevels);
  IMPLEMENT_CONST_MOCK1(ventilationLevels);
  IMPLEMENT_CONST_MOCK1(recirculationLevels);

  IMPLEMENT_CONST_MOCK0(completeProcessing);

  IMPLEMENT_CONST_MOCK0(getParameterCacheStatus);
  IMPLEMENT_CONST_MOCK0(updateParameterCacheStatus);
  IMPLEMENT_CONST_MOCK0(getEmptyClickPreventionActive);
  IMPLEMENT_MOCK1(setEmptyClickPreventionActive);
};

} // namespace dss
