/*
    Copyright (c) 2016 digitalSTROM.org," " Zurich," " Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/
#include "signals-source.h"

#include <ds-model/model.h>
#include <ds/asio/ds-signal-mock.h>
#include <ds/asio/test.h>
#include <event.h>
#include <event/event_create.h>
#include <model/apartment-mock.h>
#include <model/zone.h>

using namespace std::literals;
static ds::LogChannel dsLogChannel("dsBackendSignalsSourceTest");
static const char* TAGS = "[dss][dsBackendSignalsSource]";

namespace {

DS_SLOT_MOCK1(SignalSlotMock, dsModel::Signal);

TEST_CASE("dsBackendSignalsSource", TAGS) {
  ds::asio::TestLoopResource loop;
  dss::EventInterpreter eventInterpreter{nullptr};
  dss::EventQueue queue{&eventInterpreter};
  eventInterpreter.initialize();
  eventInterpreter.setEventQueue(&queue);
  auto source = dsBackend::makeSignalsSource(loop, eventInterpreter);
  auto nextSignal = SignalSlotMock{loop, source->next()};

  dss::ApartmentMock apartment{};
  auto& zone0 = *apartment.allocateZone(0);
  const auto lastChanged = loop.system.now() - 1min;

  SECTION("Apartment scenes emitted") {
    struct X {
      int dssScene{};
      bool force{};
      dsModel::ApartmentActionType dsAction{};
    };
    const auto& x = GENERATE(values(std::initializer_list<X>{
        {.dssScene = dss::SceneAbsent, .force = false, .dsAction = dsModel::ApartmentActionType::absent},
        {.dssScene = dss::SceneAbsent, .force = true, .dsAction = dsModel::ApartmentActionType::absent},
        {.dssScene = dss::ScenePresent, .force = false, .dsAction = dsModel::ApartmentActionType::present},
        {.dssScene = dss::ScenePresent, .force = true, .dsAction = dsModel::ApartmentActionType::present},
    }));
    INFO(x.dssScene);
    INFO(x.dsAction);
    REQUIRE_CALL(loop.system, randomUuidString()).RETURN(ds::str("a"));
    REQUIRE_CALL(nextSignal,
                 call(dsModel::Signal{
                     .id = dsModel::SignalId{"a"},
                     .type = "signal",
                     .lastChanged = lastChanged,
                     .attributes = {.target = dsModel::SignalTargetApartment{.type = "apartment"},
                                    .action = dsModel::ApartmentAction{.type = "apartment", .id = x.dsAction}}}))
        .LR_SIDE_EFFECT(loop.stop());
    queue.pushEvent([&] {
      auto result = dss::createGroupCallSceneEvent(zone0.getGroup(0), x.dssScene, 0, 0, {}, {}, {}, x.force);
      result->setTimestamp(dss::DateTime{lastChanged});
      return result;
    }());
    eventInterpreter.executePendingEvent();
    loop.run();
  }
}

} // namespace
