/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/asio/resource.h>

#include "message-stream.h"

namespace dsNet {

struct BufferedMessageWriterConfig {
  size_t countCapacity = 0;
  size_t sizeCapacity = 0;
};

struct BufferedMessageWriterHooks {
  /// Background write failed.
  /// Recovery is not possible, BufferedMessageWriter needs to be recreated.
  std::function<void(const std::exception&)> failed = [](auto&&) {};
  std::function<void()> enqueueWritePostponed = []() {};
};

/// Writer buffering messages in queue and writing to stream on background.
struct BufferedMessageWriter : virtual ds::asio::IResource<BufferedMessageWriterConfig>, virtual dsNet::ISession {
  virtual size_t count() const = 0;
  virtual size_t size() const = 0;

  /// Enqueue message for write.
  /// Synchronous errors are propagated by exception,
  /// asynchronous by failed hook.
  virtual void enqueue(std::string&& message) = 0;
};
dsNet::ClosePtr<BufferedMessageWriter> makeBufferedMessageWriter(ds::asio::ResourceBase&, dsNet::MessageStream&,
                                                                 BufferedMessageWriterConfig&&,
                                                                 BufferedMessageWriterHooks&&);

} // namespace dsNet

DS_JSON_DECLARE_STRUCT((dsNet), BufferedMessageWriterConfig, (countCapacity)(sizeCapacity));
