#include "ssl.h"

namespace asio = boost::asio;
using namespace std::literals;

namespace dsNet {

boost::asio::ssl::context makeSslServerContext(const std::string& sslCertificateFile) {
  // It is too expensive to generate DH key for each session.
  // Ideally, DH key is unique for each device and sometimes rotated.
  // It would prevent sharing the precomputation effort of an attack on DH.
  const auto dh = R"(
-----BEGIN DH PARAMETERS-----
MIIBCAKCAQEAlXG+SNv7pLl8+8TBZRqvSVxx8QdqD0cnpKdYFB0ZDDZLfxxeSJ38
IvwGN7y22MgAAe9IUCS+hKqFZ0lvCI0RFaIf27Ueiof26qNx6obkFfJH0dzxtNaA
E6PVXGgnpWWPg4h7go057N9GzqBXyjz6/TwgWQH3k6b2jxwmRZhOMjAVCm9nX3cN
S6M5jmdWjWktjfoduzfME6DSaOJE+rq3eRQXPSvQFIH1F0xuDto7vYEaavIbBcCM
UX46RU8saLDZ8GUIri/X2x0gFsJTMVJnASix4kJS8vZ4sP8mWEr2EGXBpf2ZUK92
J+zrv+F+73KA3jOcjYFX/Kz1GKmHx5mbQwIBAg==
-----END DH PARAMETERS-----
)"sv;
  auto result = asio::ssl::context{asio::ssl::context::sslv23};
  result.set_options(asio::ssl::context::default_workarounds | asio::ssl::context::no_sslv2 |
                     asio::ssl::context::single_dh_use);
  result.use_certificate_chain_file(sslCertificateFile);
  result.use_private_key_file(sslCertificateFile, asio::ssl::context::file_format::pem);
  result.use_tmp_dh(boost::asio::buffer(dh.data(), dh.size()));
  return result;
}

} // namespace dsNet
