/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/asio/resource.h>
#include <ds/clock.h>
#include <ds/reflect.h>

namespace ds::asio {

using namespace std::literals;

struct BackOffConfig {
  ds::Clock::duration first{2s};
  ds::Clock::duration max{30min};
};

extern template class ds::asio::Resource<BackOffConfig>;

class BackOff : public ds::asio::Resource<BackOffConfig> {
public:
  using ds::asio::Resource<BackOffConfig>::Resource;

  /// Reset the back off. Next call to `next()` will return 0.
  void reset() { m_next = {}; }

  ds::Clock::duration peek() const { return m_next; }
  ds::Clock::duration next();

private:
  ds::Clock::duration m_next{};
};

} // namespace ds::asio

DS_JSON_DECLARE_STRUCT((ds)(asio), BackOffConfig, (first)(max));
