/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/clock.h>
#include <ds/conf.h>
#include <boost/asio/io_service.hpp>
#include "latency-watchdog.h"
#include "resource.h"

namespace ds::asio {

//
// Event loop implemneted as ds::asio::io_service running in one thread.
//
class Loop {
public:
  Loop(ds::Conf &conf, std::string &&name, LatencyWatchdogConfig &&);
  ~Loop();

  boost::asio::io_service &io();
  ResourceContext &context() { return m_context; }
  operator ds::asio::ResourceBase &() { return m_context; }

  template <class F>
  auto invoke(F &&f) const {
    return m_context.invoke(std::forward<F>(f));
  }

private:
  struct Impl;
  const std::unique_ptr<Impl> m_impl;
  ResourceContext m_context;
};

} // namespace ds::asio
