/*
    Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "watchdog.h"
#include <ds/log.h>
#include <atomic>
#include <boost/asio.hpp>
#include <condition_variable>
#include <mutex>
#include <thread>

#include <execinfo.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

static ds::LogChannel dsLogChannel("dsAsioWatchdog");

namespace ds::asio {

FileDescriptorsWatchdog::FileDescriptorsWatchdog(boost::asio::io_service &io, std::chrono::milliseconds period)
    : m_timer(io), m_period(period), m_fire([this](const std::exception &e) { defaultFire(e); }) {
  DS_INFO_ENTER(m_period.count());
  asyncLoop();
}

FileDescriptorsWatchdog::~FileDescriptorsWatchdog() = default;

void FileDescriptorsWatchdog::asyncLoop() {
  m_timer.randomlyExpiresFromNowPercentDown(m_period, 20, [this] {
    DS_INFO_ENTER();
    try {
      auto &io = m_timer.ioService();
      boost::asio::local::stream_protocol::socket socket1(io);
      boost::asio::local::stream_protocol::socket socket2(io);
      boost::asio::local::connect_pair(socket1, socket2);
    } catch (const std::exception &e) {
      DS_INFO("Fire.", e);
      m_fire(e);
    }
    asyncLoop();
  });
}

void FileDescriptorsWatchdog::defaultFire(const std::exception &e) {
  DS_FAIL_ASSERT("FileDescriptorsWatchdog failed.", e);
}

} // namespace ds::asio
