#include "conf.h"
#include "memory.h"

namespace ds {

namespace detail {
LogChannel confLogChannel("dsConf");
} // namespace detail

static ds::LogChannel& dsLogChannel = detail::confLogChannel;

class ConfImpl final : public Conf {
public:
  ConfImpl(System& system) : m_system(system) {}

  std::optional<JsonFragment> readJsonIfExists(std::string_view key) const override {
    DS_DEBUG_ENTER(key);
    const auto& filename = ds::str(m_system.dirs().sysconfdir, "/ds-conf/", key, ".json");
    if (const auto& fileContent = m_system.readFileIfExists(filename)) {
      DS_INFO_LEAVE(key, (*fileContent));
      return ds::fromJson<JsonFragment>(*fileContent);
    }
    return {};
  }

  void writeJson(std::string_view key, const std::string &json) const override {
    DS_DEBUG_ENTER(key, json);
    const auto& dsConfDir = ds::str(m_system.dirs().sysconfdir, "/ds-conf/");
    try {
      if (!boost::filesystem::is_directory(dsConfDir)) {
        boost::filesystem::create_directory(dsConfDir);
      }
    } catch (std::exception& e) {
      DS_ERROR("Cannot create directory. ", dsConfDir);
      return;
    }
    m_system.writeToFile(ds::str(dsConfDir, key, ".json"), json);
  }

private:
  System& m_system;
};

LogChannel& Conf::logChannel() { return dsLogChannel; }
std::unique_ptr<Conf> makeConf(System& system) {
  return std::make_unique<ConfImpl>(system);
}

} // namespace ds
