/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "expected.h"

#include <ds/catch/catch.h>

namespace {

static const char* TAGS = "[dsExpected][ds]";

TEST_CASE("dsExpected", TAGS) {
  GIVEN("value initialized") {
    auto x = ds::Expected<int>(3);

    THEN("value()") { CHECK(x.value() == 3); }
    THEN("operator *()") { CHECK(*x == 3); }
    THEN("operator bool") { CHECK(x); }
    THEN("operator ==, !=") {
      CHECK(x == 3);
      CHECK(x != 4);
    }
    THEN("value_or()") { CHECK(x.value_or(-1) == 3); }
    THEN("has_value()") { CHECK(x.has_value()); }
    THEN("map") {
      CHECK(x.map([](auto&& y) { return ds::str(y); }) == "3");
    }
    THEN("and_then with value") {
      CHECK(x.and_then([](auto&& y) -> ds::Expected<std::string> { return ds::str(y); }) == "3");
    }
    THEN("and_then with error") {
      CHECK(!x.and_then([](auto &&) -> ds::Expected<std::string> { return DS_UNEXPECTED("test error"); }));
    }
    THEN("operator std::optional<T>") {
      CHECK([&]() -> std::optional<int> { return std::move(x); }() == 3);
    }
  }
  GIVEN("error initialized") {
    auto x = ds::Expected<int>{DS_UNEXPECTED("test error")};

    THEN("value()") { CHECK_THROWS_FIND(x.value(), "test error"); }
    THEN("operator *()") { CHECK_THROWS_FIND(*x, "test error"); }
    THEN("operator bool") { CHECK(!bool(x)); }
    THEN("operator ==, !=") { CHECK(x != 3); }
    THEN("value_or()") { CHECK(x.value_or(-1) == -1); }
    THEN("has_value()") { CHECK(!x.has_value()); }
    THEN("map") {
      CHECK(!x.map([](auto&& y) -> std::string {
        CHECK(false);
        return {};
      }));
    }
    THEN("and_then") {
      CHECK(!x.and_then([](auto&& y) -> ds::Expected<std::string> {
        CHECK(false);
        return {};
      }));
    }
    THEN("operator std::optional<T>") {
      CHECK([&]() -> std::optional<int> { return std::move(x); }() == std::nullopt);
    }
  }
}
} // namespace
