/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "latency-statistics.h"
#include <ds/catch/catch.h>
#include <ds/str.h>

namespace {

static const char* TAGS = "[dsLatencyStatistics][ds]";

TEST_CASE("dsLatencyStatistics", TAGS) {
  ds::LatencyStatistics x;

  SECTION("empty") {
    CHECK(x.count() == 0);
    CHECK(ds::str(x) == "min:1.79769e+308 avg:-nan max:-1.79769e+308 sdev:-nan count:0");
  }

  SECTION("with 3") {
    x(3);

    CHECK(x.count() == 1);
    CHECK(x.min() == 3);
    CHECK(x.max() == 3);
    CHECK(x.avg() == 3);
    CHECK(x.sdev() == 0);
    CHECK(ds::str(x) == "min:3 avg:3 max:3 sdev:0 count:1");
  }

  SECTION("with 3,7") {
    x(3);
    x(7);

    CHECK(x.count() == 2);
    CHECK(x.min() == 3);
    CHECK(x.max() == 7);
    CHECK(x.avg() == 5);
    CHECK(x.sdev() == 2);
    CHECK(ds::str(x) == "min:3 avg:5 max:7 sdev:2 count:2");
  }

  SECTION("drop works") {
    x(3);
    x.drop();
    x(7);

    CHECK(x.count() == 1);
    CHECK(x.min() == 7);
    CHECK(x.max() == 7);
    CHECK(x.avg() == 7);
    CHECK(x.sdev() == 0);
    CHECK(ds::str(x) == "min:7 avg:7 max:7 sdev:0 count:1");
  }
}

} // namespace
