/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <iosfwd>
#include <memory>

namespace ds {

/// Accumulates latency samples and provide useful latency statistics
class LatencyStatistics {
public:
  LatencyStatistics();
  ~LatencyStatistics();

  /// add latency sample
  void operator()(double x);

  int count() const;
  double min() const;
  double avg() const;
  double max() const;
  double sdev() const; ///< standard deviation

  /// drop all statistics and start again
  void drop();

private:
  struct Impl;
  std::unique_ptr<Impl> m_impl;
};

/// Print all statistics in human readable form
std::ostream &operator<<(std::ostream &ostream, const LatencyStatistics &x);

} // namespace ds
