/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include "ds485-device.h"

#include <ds/catch/catch.h>
#include <ds/log.h>

#include "dsm-api-mock.h"
#include "event-mock.h"
#include "model/apartment-mock.h"
#include "model/device-mock.h"
#include "model/modelmaintenance-mock.h"
#include "propertysystem-mock.h"

static ds::LogChannel dsLogChannel("ds485DeviceTest");
static const char* TAGS = "[ds485][device]";

using namespace dss;
using namespace std::literals;
using namespace trompeloeil;
using trompeloeil::_;

namespace {

SCENARIO("Receive ModelEvents", TAGS) {
  PropertySystemMock propertySystem{};
  EventSinkMock eventSink;
  ModelMaintenanceMock modelMaintenance{propertySystem, eventSink};

  DSDeviceBusInterface dsDeviceBusInterface{};
  dsDeviceBusInterface.setDSMApiHandle(&dsm_api_mock);

  dsDeviceBusInterface.setModelMaintenace(&modelMaintenance);

  ApartmentMock apartment{};
  auto& device = *createDevice(apartment, {.hardwareType = HardwareType::GR_KL300});

  SECTION("setValue") {
    REQUIRE_CALL(dsm_api_mock, DeviceActionRequest_action_set_outval(_, _, _, _)).RETURN(0);
    REQUIRE_CALL(modelMaintenance, addModelEvent(_))
        .LR_SIDE_EFFECT(CHECK(_1->getEventType() == ModelEvent::etDeviceSetOutputValue));
    dsDeviceBusInterface.setValue(device, 0x7f);
  }

  SECTION("getSensorValue") {
    REQUIRE_CALL(dsm_api_mock, DeviceSensor_get_value_sync(_, _, _, _, _, _)).RETURN(0);
    REQUIRE_CALL(modelMaintenance, addModelEvent(_))
        .LR_SIDE_EFFECT(CHECK(_1->getEventType() == ModelEvent::etDeviceSensorValue));
    dsDeviceBusInterface.getSensorValue(device, 0x2);
  }

  SECTION("getSensorValueEx") {
    REQUIRE_CALL(dsm_api_mock, DeviceSensor_get_value_extended_sync(_, _, _, _, _, _, _, _, _)).RETURN(0);
    REQUIRE_CALL(modelMaintenance, addModelEvent(_))
        .LR_SIDE_EFFECT(CHECK(_1->getEventType() == ModelEvent::etDeviceSensorValueEx));
    dsDeviceBusInterface.getSensorValueEx(device, 0x2);
  }
}

} // namespace
