/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DSSTRUCTUREMODIFYINGBUSINTERFACE_H_
#define DSSTRUCTUREMODIFYINGBUSINTERFACE_H_

#include "businterface.h"
#include "ds485/dsbusinterfaceobj.h"

namespace dss {

class ModelMaintenance;

class DSStructureModifyingBusInterface : public DSBusInterfaceObj, public StructureModifyingBusInterface {
public:
  DSStructureModifyingBusInterface() : DSBusInterfaceObj(), m_pModelMaintenance(NULL) {}

  std::string name() const override;

  void setZoneID(const dsuid_t& _dsMeterID, const devid_t _deviceID, const int _zoneID) override;
  void createZone(const dsuid_t& _dsMeterID, const int _zoneID) override;
  void removeZone(const dsuid_t& _dsMeterID, const int _zoneID) override;

  void addToGroup(const dsuid_t& _dsMeterID, const int _groupID, const int _deviceID) override;
  void removeFromGroup(const dsuid_t& _dsMeterID, const int _groupID, const int _deviceID) override;
  void removeDeviceFromDSMeter(const dsuid_t& _dsMeterID, const int _deviceID) override;
  void removeDeviceFromDSMeters(const dsuid_t& _DeviceDSID) override;

  void sceneSetName(uint16_t _zoneID, uint8_t _groupID, uint8_t _sceneNumber, const std::string& _name) override;
  void deviceSetName(dsuid_t _meterDSID, devid_t _deviceID, const std::string& _name) override;
  void meterSetName(dsuid_t _meterDSID, const std::string& _name) override;

  void createGroup(uint16_t _zoneID, uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig,
                   const std::string& _name) override;
  void removeGroup(uint16_t _zoneID, uint8_t _groupID) override;
  void groupSetApplication(uint16_t _zoneID, uint8_t _groupID, ApplicationType applicationType,
                           uint32_t applicationConfig) override;
  void groupSetName(uint16_t _zoneID, uint8_t _groupID, const std::string& _name) override;

  void createCluster(uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig,
                     const std::string& _name) override;
  void removeCluster(uint8_t _clusterID) override;
  void clusterSetName(uint8_t _clusterID, const std::string& _name) override;
  void clusterSetApplication(uint8_t _clusterID, ApplicationType applicationType, uint32_t applicationConfig) override;
  void clusterSetProperties(uint8_t _clusterID, uint16_t _location, uint16_t _floor,
                            uint16_t _protectionClass) override;
  void clusterSetLockedScenes(uint8_t _clusterID, const std::vector<int>& lockedScenes) override;
  void clusterSetConfigurationLock(uint8_t _clusterID, bool _lock) override;
  void clusterSetOperationLock(uint8_t _clusterID, bool _lock) override;

  void setButtonSetsLocalPriority(const dsuid_t& _dsMeterID, const devid_t _deviceID, bool _setsPriority) override;
  void setButtonCallsPresent(const dsuid_t& _dsMeterID, const devid_t _deviceID, bool _callsPresent) override;

  void setZoneHeatingConfig(const dsuid_t& _dsMeterID, const uint16_t _ZoneID,
                            const ZoneHeatingConfigSpec_t _spec) override;
  void setZoneHeatingOperationModes(const dsuid_t& _dsMeterID, const uint16_t _ZoneID,
                                    const ZoneHeatingOperationModeSpec_t _spec) override;
  void setZoneSensor(const uint16_t _zoneID, SensorType sensorType, const dsuid_t& sensorDSUID) override;
  void resetZoneSensor(const uint16_t _zoneID, SensorType sensorType) override;

  void setCircuitPowerStateConfig(const dsuid_t& _dsMeterID, const int _index, const int _setThreshold,
                                  const int _resetThreshold) override;
  void setMeteringConfiguration(const dsuid_t& _dsMeterID, const MeteringConfigurationSpec_t& _spec) override;

  void setModelMaintenace(ModelMaintenance* _modelMaintenance);

  void setProperty(const dsuid_t& _meter,
                   const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& properties) override;
  vdcapi::Message getProperty(const dsuid_t& _meter,
                              const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& query) override;

  void setFireMuteEnabled(dsuid_t dsm, bool) override;

  void removeVdc(const dsuid_t& vdsmID, const dsuid_t& vdcID) override;

  void clusterSetStopTimeout(uint8_t cluster, uint8_t timeoutSeconds) override;

  void zoneSetStopTimeout(uint16_t zone, uint8_t group, uint8_t timeoutSeconds) override;

private:
  ModelMaintenance* m_pModelMaintenance;
}; // DSStructureModifyingBusInterface

} // namespace dss

#endif
