/*
 *  Copyright (c) 2012 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "dssfwd.h"
#include "model/deviceinterface.h"
#include "system_handlers.h"

namespace dss {

class EventLogger {
public:
  EventLogger(const std::string& _filePath, const std::string& _logname);
  virtual ~EventLogger();
  void log(const std::string& text);
  void logln(const std::string& text);
  const std::string& getLogName() const { return m_logName; }
  void reopenLogfile();

private:
  std::string m_logName;
  FILE* m_f;
  boost::mutex m_LogWriteMutex;
  std::string m_fileName;
};

class SystemEventLog : public SystemEvent {
public:
  SystemEventLog(const Event& event);
  virtual ~SystemEventLog();
  void run() override;

  // unit tests
  std::string getZoneName(std::shared_ptr<Zone> _zone);
  std::string getGroupName(std::shared_ptr<Group> _group);
  std::string getSceneName(int _sceneID, int _groupID);
  std::string getDeviceName(std::string _orgin_device_id);
  std::string getCallOrigin(CallOrigin _call_origin);

private:
  void logDeviceLocalScene(std::shared_ptr<EventLogger> _logger, int _scene_id, std::string _origin_dsid,
                           CallOrigin _call_origin);
  void logZoneGroupScene(std::shared_ptr<EventLogger> _logger, std::shared_ptr<Zone> _zone, int _group_id,
                         int _scene_id, bool _is_forced, std::string _origin_dsid, CallOrigin _call_origin,
                         std::string _origin_token);
  void logDeviceScene(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device,
                      std::shared_ptr<Zone> _zone, int scene_id, bool _is_forced, std::string _origin_dsid,
                      CallOrigin _call_origin, std::string _token);
  void logDeviceUndo(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device,
                     std::shared_ptr<Zone> _zone, int _scene_id, std::string _origin_dsuid, CallOrigin _call_origin,
                     std::string _token);

  void logLastScene(std::shared_ptr<EventLogger> _logger, std::shared_ptr<Zone> _zone,
                    std::shared_ptr<Group> _group, int _scene_id);
  void logZoneGroupBlink(std::shared_ptr<EventLogger> _logger, std::shared_ptr<Zone> _zone, int _group_id,
                         std::string _origin_dsid, CallOrigin _call_origin, std::string _origin_token);
  void logDeviceBlink(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device,
                      std::shared_ptr<Zone> _zone, std::string _origin_device_id, CallOrigin _call_origin,
                      std::string _token);
  void logZoneGroupUndo(std::shared_ptr<EventLogger> _logger, std::shared_ptr<Zone> _zone, int _group_id,
                        int _scene_id, std::string _origin_dsid, CallOrigin _call_origin, std::string _origin_token);
  void logDeviceButtonClick(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDirectDeviceAction(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDeviceNamedAction(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDeviceNamedEvent(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDeviceNamedState(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDeviceBinaryInput(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDeviceSensorEvent(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device);
  void logDeviceSensorValue(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const Device> _device,
                            std::shared_ptr<Zone> _zone);
  void logZoneSensorValue(std::shared_ptr<EventLogger> _logger, std::shared_ptr<Zone> _zone, int _groupId);
  void logStateChange(std::shared_ptr<EventLogger> _logger, std::shared_ptr<const State> _st,
                      const std::string& _statename, const std::string& _state, const std::string& _value,
                      const std::string& _origin_device_id, const CallOrigin _callOrigin);
  void logSunshine(std::shared_ptr<EventLogger> _logger, std::string _value, std::string _direction,
                   std::string _originDeviceID);
  void logFrostProtection(std::shared_ptr<EventLogger> _logger, std::string _value, std::string _originDeviceID);
  void logHeatingModeSwitch(std::shared_ptr<EventLogger> _logger, std::string _value, std::string _originDeviceID);
  void logBuildingService(std::shared_ptr<EventLogger> _logger, std::string _value, std::string _originDeviceID);
  void logExecutionDenied(std::shared_ptr<EventLogger> _logger, std::string _action, std::string _reason);
  void logOperationLock(std::shared_ptr<EventLogger> _logger, std::shared_ptr<Zone> _zone, int _groupId, int _lock,
                        CallOrigin _call_origin);
  void logDevicesFirstSeen(std::shared_ptr<EventLogger> _logger, std::string& dateTime, std::string& token,
                           CallOrigin _call_origin);
  void logHighLevelEvent(std::shared_ptr<EventLogger> _logger, std::string& _id, std::string& _name);

  bool canLogSensor(SensorType _sensorType);

  void model_ready();
  void callScene();
  void blink();
  void undoScene();
  void buttonClick();
  void directDeviceAction();
  void deviceNamedAction();
  void deviceNamedEvent();
  void deviceNamedState();
  void deviceBinaryInputEvent();
  void deviceSensorEvent();
  void deviceSensorValue();
  void zoneSensorValue();
  void stateChange();
  void sunshine();
  void frostProtection();
  void heatingModeSwitch();
  void buildingService();
  void executionDenied();
  void operationLock(const std::string& _evtName);
  void devicesFirstSeen();
  void highlevelevent();
};
}; // namespace dss
