/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "archives-connection.h"

#include <ds/catch/catch.h>
#include <ds/system-mock.h>

#include <trompeloeil.hpp>

#include "archives-connection-mock.h" // RRDInterfaceMock

using namespace trompeloeil;
using namespace std::literals;

using namespace dss;
using namespace dss::metering;

using system_clock = std::chrono::system_clock;

static ds::LogChannel dsLogChannel("dssMeteringArchivesConnectionTest");

namespace {

static const char* TAGS = "[dSS][metering][archivesConnection]";

SCENARIO("archiveConnection-interface", TAGS) {
  auto rrdInterface = std::make_shared<RRDInterfaceMock>();
  auto system = ds::SystemMock();

  auto archivesConnection = makeArchivesConnection(rrdInterface, system);

  GIVEN("rrdCachedPath") {
    WHEN("not set") { CHECK(archivesConnection->rrdCachedPath().empty()); }
    WHEN("set") {
      archivesConnection->setRRDCachedPath("unix:cache-path"s);
      CHECK(archivesConnection->rrdCachedPath() == "unix:cache-path"s);
    }
  }
  GIVEN("lastTimestamp") {
    WHEN("failure") {
      REQUIRE_CALL(*rrdInterface, lastTimestamp(_)).RETURN(std::nullopt);
      CHECK(archivesConnection->lastTimestamp("x.rrd"s) == std::nullopt);
    }
    WHEN("success") {
      REQUIRE_CALL(*rrdInterface, lastTimestamp(_)).RETURN(300);
      CHECK(archivesConnection->lastTimestamp("x.rrd"s) == ds::DateTime{300s});
    }
  }
  SECTION("flush") {
    WHEN("no cached") {
      CHECK_NOTHROW(archivesConnection->flush({"x.rrd"s}));
    }
    WHEN("with cached") {
      THEN("flush is executed") {
        archivesConnection->setRRDCachedPath("unix:socket"s);
        REQUIRE_CALL(*rrdInterface,
                     flush(std::vector{"flushcached"s, "--daemon"s, "unix:socket"s, "x.rrd"s, "y.rrd"s, "z.rrd"s}));
        CHECK_NOTHROW(archivesConnection->flush({"x.rrd"s, "y.rrd"s, "z.rrd"s}));
      }
    }
  }
}

} // namespace
