/*
    Copyright (c) 2009,2010,2011 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <ds/date-time.h>
#include <ds/system.h>

#include <optional>
#include <string>
#include <vector>

namespace dss::metering {

class ArchivesStorage;
class RRDInterface;

class ArchivesConnection {
public:
  virtual ~ArchivesConnection() = default;

  virtual void flush(const std::vector<std::string>& paths) const = 0;
  virtual void restartCached() const = 0;
  virtual void setRRDCachedPath(std::string rrdCachedPath) = 0;

  virtual std::optional<ds::DateTime> lastTimestamp(const std::string& path) const = 0;

  virtual ArchivesStorage& getArchivesStorage() const = 0;

  // transitional, hide later
  virtual RRDInterface& rrdInterface() const = 0;
  virtual std::string rrdCachedPath() const = 0;
};

std::shared_ptr<ArchivesConnection> makeArchivesConnection(std::shared_ptr<RRDInterface> rrdInterface,
                                                           ds::System& system);

} // namespace dss::metering
