/*
    Copyright (c) 2009,2010,2011 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <ds/clock.h>
#include <ds/json.h>

using namespace std::literals;

namespace dss::metering {

struct DiscontinuityDetection {
  ds::Clock::duration interval = 3s;
  int maxConsumption = 8'000;    // 2 * 16A * 250V, #10705
  int maxProduction = 50'000;    // 3 big solar panels (15 kW each)
};

struct TimeSeriesConfig {
  std::chrono::seconds resolution;
  unsigned numberOfValues;
};

struct ElectricityMeteringConfig {
  std::chrono::seconds sensorValueTimeout = 3s;
  DiscontinuityDetection discontinuityDetection{};
  std::string socketPath = "/var/run/dss-metering.sock";

  std::chrono::seconds maxInterpolationInterval =
      7 * 24h; // interpolation takes too long if gap is longer, (for 1gb / sk) DSD-xyz
  std::chrono::seconds diskFlushInterval = 10min;

  // existing archives are not migrated, you will lose data!!!
  std::vector<TimeSeriesConfig> timeSeries = {{1s, 600},  {60s, 720},     {15min, 2976},
                                              {24h, 370}, {7 * 24h, 260}, {30 * 24h, 60}};
};

} // namespace dss::metering

DS_JSON_DECLARE_STRUCT((dss::metering), DiscontinuityDetection, (interval)(maxConsumption)(maxProduction));
DS_JSON_DECLARE_STRUCT((dss::metering), TimeSeriesConfig, (resolution)(numberOfValues));
DS_JSON_DECLARE_STRUCT(
    (dss::metering), ElectricityMeteringConfig,
    (sensorValueTimeout)(discontinuityDetection)(socketPath)(maxInterpolationInterval)(diskFlushInterval)(timeSeries));
