/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <ds/json.h>
#include <ds/system-fwd.h>

#include "ds-model/meterings-fwd.h"
#include "metering/archives-fwd.h"
#include "metering/config.h"
#include "metering/metering.h" // TimeDuration

namespace dss::metering {

class OpenMeteringSystem {
public:
  using Reading = readings::OpenMeteringSystem;

  virtual ~OpenMeteringSystem() = default;

  virtual void setActiveMeterId(const OpenMeteringSystemId& id) = 0;
  virtual std::optional<OpenMeteringSystemId> activeMeterId() const = 0;
  virtual std::vector<std::pair<OpenMeteringSystemId, dsModel::MeteringType>> meterings() const = 0;
  virtual void addReading(Reading&& reading) = 0;

  virtual std::optional<LastReading> getLastReading(OpenMeteringSystemId id, dsModel::MeteringType meteringType,
                                                    EnergyUnit energyUnit) const = 0;

  // SeriesType needed for Counter vs. EnergyDelta -> add Counter to MeteringType
  virtual ValueRange getSeries(dsModel::MeteringType meteringType, SeriesTypes type, std::chrono::seconds resolution,
                               const TimeDuration& timeDuration, int valueCount, EnergyUnit energyUnit) const = 0;
};

std::unique_ptr<OpenMeteringSystem> makeOpenMeteringSystem(ArchivesConnection& archivesConnection,
                                                           ArchivesStorage& archivesStorage);

std::unique_ptr<OpenMeteringSystem> makeOpenMeteringSystemMocked(
    std::unique_ptr<archives::OpenMeteringSystemArchive> openMeteringSysteArchive, ArchivesStorage& archivesStorage,
    ds::System& system);

} // namespace dss::metering
