
/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <memory>

#include "addressablemodelitem.h"
#include "businterface-fwd.h"
#include "deviceinterface.h"
#include "ds485types.h"
#include "dssfwd.h"

namespace dss::model {

class ActionRequestExecutor {
public:
  ActionRequestExecutor(const Apartment&, ActionRequestInterface& actionRequestInterface);
  void increaseValue(AddressableModelItem& target, Caller&& caller);
  void decreaseValue(AddressableModelItem& target, Caller&& caller);
  void setValue(AddressableModelItem& target, Caller&& caller, const uint8_t _value);

  void callScene(AddressableModelItem& target, Caller&& caller, const int _sceneNr, const bool _force);

  void callSceneMin(AddressableModelItem& target, Caller&& caller, const int _sceneNr);

  void saveScene(AddressableModelItem& target, Caller&& caller, const int _sceneNr);

  void undoScene(AddressableModelItem& target, Caller&& caller, const int _sceneNr);

  void undoSceneLast(AddressableModelItem& target, Caller&& caller);

  void blink(AddressableModelItem& target, Caller&& caller);

  void increaseOutputChannelValue(AddressableModelItem& target, Caller&& caller, const ChannelType _channel);

  void decreaseOutputChannelValue(AddressableModelItem& target, Caller&& caller, const ChannelType _channel);

  void stopOutputChannelValue(AddressableModelItem& target, Caller&& caller, const ChannelType _channel);

  void pushSensor(AddressableModelItem& target, Caller&& caller, const dsuid_t _sourceID, SensorType _sensorType,
                  const double _sensorValueFloat);

  const Apartment& m_apartment;
  ActionRequestInterface& m_actionRequestInterface;
};

} // namespace dss::model
