/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#include "addressablemodelitem.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "actionrequestexecutor.h"
#include "apartment.h"
#include "businterface.h"

static ds::LogChannel dsLogChannel("modelAddressableModelItem");

namespace dss {

using dss::model::ActionRequestExecutor;

//================================================== AddressableModelItem

AddressableModelItem::AddressableModelItem(Apartment* _pApartment) : m_pApartment(_pApartment) {} // ctor

void AddressableModelItem::increaseValue(const CallOrigin _origin, const SceneAccessCategory _category) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.increaseValue(*this, {_origin, _category, ""});
} // increaseValue

void AddressableModelItem::decreaseValue(const CallOrigin _origin, const SceneAccessCategory _category) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.decreaseValue(*this, {_origin, _category, ""});
} // decreaseValue

void AddressableModelItem::setValue(const CallOrigin _origin, const SceneAccessCategory _category,
                                    const uint8_t _value, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.setValue(*this, Caller{_origin, _category, _token}, _value);
} // setValue

void AddressableModelItem::callScene(const CallOrigin _origin, const SceneAccessCategory _category,
                                     const int _sceneNr, const std::string _token, const bool _force) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.callScene(*this, {_origin, _category, _token}, _sceneNr, _force);
} // callScene

void AddressableModelItem::callSceneMin(const CallOrigin _origin, const SceneAccessCategory _category,
                                        const int _sceneNr, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.callSceneMin(*this, Caller{_origin, _category, _token}, _sceneNr);
} // callSceneMin

void AddressableModelItem::saveScene(const CallOrigin _origin, const int _sceneNr, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.saveScene(*this, Caller{_origin, SceneAccessCategory{}, _token}, _sceneNr);
} // saveScene

void AddressableModelItem::undoScene(const CallOrigin _origin, const SceneAccessCategory _category,
                                     const int _sceneNr, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.undoScene(*this, Caller{_origin, _category, _token}, _sceneNr);
} // undoScene

void AddressableModelItem::undoSceneLast(const CallOrigin _origin, const SceneAccessCategory _category,
                                         const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.undoSceneLast(*this, Caller{_origin, _category, _token});
} // undoSceneLast

void AddressableModelItem::blink(const CallOrigin _origin, const SceneAccessCategory _category,
                                 const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.blink(*this, Caller{_origin, _category, _token});
} // blink

void AddressableModelItem::increaseOutputChannelValue(const CallOrigin _origin, const SceneAccessCategory _category,
                                                      const ChannelType _channel, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.increaseOutputChannelValue(*this, Caller{_origin, _category, _token}, _channel);
} // increaseOutputChannelValue

void AddressableModelItem::decreaseOutputChannelValue(const CallOrigin _origin, const SceneAccessCategory _category,
                                                      const ChannelType _channel, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.decreaseOutputChannelValue(*this, Caller{_origin, _category, _token}, _channel);
} // decreaseOutputChannelValue

void AddressableModelItem::stopOutputChannelValue(const CallOrigin _origin, const SceneAccessCategory _category,
                                                  const ChannelType _channel, const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.stopOutputChannelValue(*this, Caller{_origin, _category, _token}, _channel);
} // stopOutputChannelValue

void AddressableModelItem::pushSensor(const CallOrigin _origin, const SceneAccessCategory _category,
                                      const dsuid_t _sourceID, SensorType _sensorType, const double _sensorValueFloat,
                                      const std::string _token) {
  ActionRequestExecutor actionRequestWrapper(*m_pApartment, *m_pApartment->getActionRequestInterface());
  actionRequestWrapper.pushSensor(*this, Caller{_origin, _category, _token}, _sourceID, _sensorType, _sensorValueFloat);
}

} // namespace dss
