/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include "deviceinterface.h"
#include "physicalmodelitem.h"

namespace dss {

class Apartment;
class PropertyNode;
typedef std::shared_ptr<PropertyNode> PropertyNodePtr;

class AddressableModelItem : public PhysicalModelItem,
                             public IDeviceInterface,
                             public std::enable_shared_from_this<AddressableModelItem> {
public:
  AddressableModelItem(Apartment* _pApartment);
  Apartment& getApartment() { return *m_pApartment; }

  void increaseValue(const CallOrigin _origin, const SceneAccessCategory _category) override;
  void decreaseValue(const CallOrigin _origin, const SceneAccessCategory _category) override;

  void setValue(const CallOrigin _origin, const SceneAccessCategory _category, const uint8_t _value,
                const std::string _token) override;

  void callScene(const CallOrigin _origin, const SceneAccessCategory _category, const int _sceneNr,
                 const std::string _token, const bool _force = false) override;
  void callSceneMin(const CallOrigin _origin, const SceneAccessCategory _category, const int _sceneNr,
                    const std::string _token) override;
  void saveScene(const CallOrigin _origin, const int _sceneNr, const std::string _token) override;
  void undoScene(const CallOrigin _origin, const SceneAccessCategory _category, const int _sceneNr,
                 const std::string _token) override;
  void undoSceneLast(const CallOrigin _origin, const SceneAccessCategory _category,
                     const std::string _token) override;

  void blink(const CallOrigin _origin, const SceneAccessCategory _category, const std::string _token) override;

  void increaseOutputChannelValue(const CallOrigin _origin, const SceneAccessCategory _category,
                                  const ChannelType channel, const std::string _token) override;
  void decreaseOutputChannelValue(const CallOrigin _origin, const SceneAccessCategory _category,
                                  const ChannelType channel, const std::string _token) override;
  void stopOutputChannelValue(const CallOrigin _origin, const SceneAccessCategory _category,
                              const ChannelType channel, const std::string _token) override;

  void pushSensor(const CallOrigin _origin, const SceneAccessCategory _category, const dsuid_t _sourceID,
                  SensorType _sensorType, const double _sensorValueFloat, const std::string _token) override;

protected:
  Apartment* m_pApartment;
  PropertyNodePtr m_pPropertyNode;
}; // AddressableModelItem
} // namespace dss
