/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include "model/device.h"
#include "structuremanipulator.h"

namespace dss {

class StructureQueryBusInterface;
class Apartment;
class DSMeter;
class Device;
class ModelMaintenance;

void setupBaseDevice(std::shared_ptr<Device> device, const DeviceSpec& spec);

class BusScanner {
public:
  BusScanner(StructureQueryBusInterface& interface, Apartment& apartment, ModelMaintenance& maintenance);
  bool scanDSMeter(std::shared_ptr<DSMeter> dsMeter);
  bool scanDeviceOnBus(std::shared_ptr<DSMeter> dsMeter, devid_t shortAddress);
  void syncBinaryInputStates(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Device> device);
  void synchronizeDSMeterData(std::shared_ptr<DSMeter> dsMeter, DSMeterSpec_t& spec);

  // for unit-tests
  bool scanZone(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Zone> zone);
  bool scanClusters(std::shared_ptr<DSMeter> dsMeter);
  bool scanPowerStates(std::shared_ptr<DSMeter> dsMeter);
  void scanDevicesOfZoneQuick(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Zone> zone);
  bool scanGroupsOfZone(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Zone> zone);
  bool scanStatusOfZone(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Zone> zone);
  bool scanTemperatureControl(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Zone> zone);
  bool initializeDeviceFromSpec(std::shared_ptr<DSMeter> dsm, std::shared_ptr<Zone> zone, const DeviceSpec& spec);

private:
  bool initializeDeviceFromSpecQuick(std::shared_ptr<DSMeter> dsMeter, std::shared_ptr<Zone> zone,
                                     const DeviceSpec& spec);
  void scheduleDeviceReadout(const std::shared_ptr<Device> pDevice);
  void setMeterCapability(std::shared_ptr<DSMeter> dsMeter);
  bool applyMeterSpec(std::shared_ptr<DSMeter> dsMeter);
  bool getMeterHash(std::shared_ptr<DSMeter> dsMeter, DSMeterHash_t& hash);
  bool getMeterState(std::shared_ptr<DSMeter> dsMeter, uint8_t* state);

private:
  Apartment& m_Apartment;
  StructureQueryBusInterface& m_Interface;
  ModelMaintenance& m_Maintenance;
  StructureManipulator m_structureManipulator;
};

} // namespace dss
