/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include "model/modelconst.h"

#include <ds/catch/catch.h>
#include <ds/log.h>

static ds::LogChannel dsLogChannel("modelModelconstTest");
static const char* TAGS = "[model][modelconst]";

using namespace dss;
using namespace std::literals;
using namespace trompeloeil;
using trompeloeil::_;

namespace {

SCENARIO("MeteringProducerCategory", TAGS) {
  CHECK(static_cast<int>(MeteringProducerCategory::consumer) == 0);
  CHECK(static_cast<int>(MeteringProducerCategory::producer) == 1);
}

TEST_CASE("SensorType", TAGS) {
  SECTION("doubleToSensorValue") {
    struct X {
      SensorType sensorType;
      double lowerBound;
      double upperBound;
    };
    const auto& x = GENERATE(values(std::initializer_list<X>{
        {SensorType::RoomTemperatureSetpoint, -43.16, 59.226},
        {SensorType::ActivePower, -1, 4096},
        {SensorType::OutputCurrent, -1, 4096.32},
        {SensorType::ElectricMeter, -0.25, 41.265},
        {SensorType::TemperatureIndoors, -44, 60},
        {SensorType::TemperatureOutdoors, -44, 59.524},
        {SensorType::BrightnessIndoors, 0, 132000.254},
        {SensorType::BrightnessOutdoors, -5698, 200000.326598},
        {SensorType::HumidityIndoors, -1, 103.264},
        {SensorType::HumidityOutdoors, -0.1, 102.4},
        {SensorType::AirPressure, 199, 1224},
        {SensorType::WindSpeed, -4, 105},
        {SensorType::WindDirection, -1, 512},
        {SensorType::Precipitation, -1, 102.5},
        {SensorType::CO2Concentration, 0, 132000},
        {SensorType::RoomTemperatureControlVariable, -101, 104},
        {SensorType::OutputCurrent16A, -15, 16381},
        {SensorType::ActivePowerVA, -1, 4096},
    }));
    INFO(ds::str(x.sensorType, " lower:", x.lowerBound));
    CHECK_THROWS(doubleToSensorValue(x.sensorType, x.lowerBound));
    // CHECK_NOTHROW(doubleToSensorValue(x.sensorType, x.lowerBound + 1));
    INFO(ds::str(x.sensorType, " upper:", x.upperBound));
    CHECK_THROWS(doubleToSensorValue(x.sensorType, x.upperBound));
    // CHECK_NOTHROW(doubleToSensorValue(x.sensorType, x.upperBound - 1));
    auto middle = [](const X& x) -> double { return (x.upperBound + x.lowerBound) / 2; };
    INFO(ds::str(x.sensorType, " middle:", middle(x)));
    CHECK_NOTHROW(doubleToSensorValue(x.sensorType, middle(x)));
  }

  SECTION("print SensorType") {
    CHECK(ds::str(SensorType::ElectricMeter) == "Electric Meter"s);
    CHECK(ds::str(SensorType::ActivePower) == "Active Power"s);
    CHECK(sensorTypeName(SensorType::ActivePower) == "Active Power"s);
  }
}

}; // namespace
