/*
    Copyright (c) 2018 digitalSTROM AG, Zurich, Switzerland

    This file is part of dSS.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dSS. If not, see <http://www.gnu.org/licenses/>.
*/

#include "modelmaintenance.h"

#include <trompeloeil.hpp>

#include "apartment.h"
#include "ds/catch/catch.h"
#include "ds/log.h"
#include "dss.h"
#include "event-mock.h"
#include "modelmaintenance-mock.h"
#include "propertysystem-mock.h"
#include "webservice_connection-mock.h"

static ds::LogChannel dsLogChannel("modelMaintenanceTest", ds::LogSeverity::DEBUG);
static const char* TAGS = "[dss][modelMaintenance]";

using namespace dss;
using namespace trompeloeil;
using trompeloeil::_;

static const Dsuid dsuid1{"0000000000000000000000000000000001"};

namespace {

SCENARIO("ModelMaintenance", TAGS) {
  PropertySystemMock propertySystem{};
  EventSinkMock eventSink{};
#if 0
  GIVEN("a OEM webquery") {
    auto pApartment = makeApartment(nullptr, &propertySystem);
    auto& apt = *pApartment;
    auto modelMaintenance = std::make_shared<ModelMaintenanceMock>(propertySystem, eventSink);
    apt.setModelMaintenance(modelMaintenance.get());
    DSS::getInstance()->setModelMaintenance(modelMaintenance);
    DS_DEFER(DSS::shutdown());
    WebserviceConnectionMock conn;
    std::shared_ptr<Device> dev1 = apt.allocateDevice(dsuid1);

    ALLOW_CALL(*modelMaintenance, addModelEvent(_)).LR_SIDE_EFFECT(delete _1);

    WHEN("the EAN is 13 characters") {
      dev1->setOemInfo(7477191559035, 0, 1, DEVICE_OEM_EAN_NO_INTERNET_ACCESS, true);
      auto task =
          std::make_shared<OEMWebQuery>(&conn, dev1, dev1->getOemProductInfoState(), propertySystem);
      THEN("the 13 characters are returned") {
        REQUIRE_CALL(propertySystem, getStringValue("/system/host/interfaces/eth0/mac")).RETURN("00:00:00:00:00:01");
        REQUIRE_CALL(propertySystem, getStringValue("/config/geodata/country")).RETURN("ch");
        REQUIRE_CALL(propertySystem, getStringValue("/system/language/locale")).RETURN("de_CH");
        REQUIRE_CALL(conn, request(_,
                                   "ean=7477191559035&partNr=1&oemSerialNumber=0&macAddress=00:00:00:00:00:01&"
                                   "countryCode=ch&languageCode=de",
                                   _, _, _));
        task->run();
      }
    }
    WHEN("the EAN is 12 characters") {
      dev1->setOemInfo(747719155903, 0, 1, DEVICE_OEM_EAN_NO_INTERNET_ACCESS, true);
      auto task =
          std::make_shared<OEMWebQuery>(&conn, dev1, dev1->getOemProductInfoState(), propertySystem);
      THEN("the 13 characters are returned") {
        REQUIRE_CALL(propertySystem, getStringValue("/system/host/interfaces/eth0/mac")).RETURN("00:00:00:00:00:01");
        REQUIRE_CALL(propertySystem, getStringValue("/config/geodata/country")).RETURN("us");
        REQUIRE_CALL(propertySystem, getStringValue("/system/language/locale")).RETURN("fr_CH");
        REQUIRE_CALL(conn, request(_,
                                   "ean=0747719155903&partNr=1&oemSerialNumber=0&macAddress=00:00:00:00:00:01&"
                                   "countryCode=us&languageCode=fr",
                                   _, _, _));
        task->run();
      }
    }
  }
#endif
}

} // namespace
