/*
    Copyright (c) 2010,2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include <digitalSTROM/dsuid.h>

#include <memory>
#include <mutex>

#include "dssfwd.h"
#include "thread.h"

namespace dss {
class DSS;
class StructureQueryBusInterface;
class StructureModifyingBusInterface;

class MeterMaintenance : public Thread {
public:
  MeterMaintenance(DSS* _pDSS, const std::string& _name);
  virtual void shutdown() { Thread::terminate(); }
  bool isRunning() { return Thread::isRunning(); }
  void triggerMeterSynchronization();
  /** Starts the event-processing */
  virtual void execute();

private:
  void synchronizeMeters();
  int getNumValiddSMeters() const;
  void readOutPendingMeter();
  void dsMeterReady(const dsuid_t& _dsMeterBusID);
  void setApartmentState();
  void raiseEvent(const std::shared_ptr<Event>& event);
  void setupInitializedState();
  void monitorInitialization();
  int getdSMBusMemberCount();

private:
  DSS* m_pDSS;
  Apartment* m_pApartment;
  StructureModifyingBusInterface* m_pModifyingBusInterface;
  StructureQueryBusInterface* m_pQueryBusInterface;
  bool m_IsInitializing;
  std::mutex m_syncMutex;
  bool m_triggerSynchronize;
  int m_retryCount;
};

} // namespace dss
